﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Threading;
using UCommerce;
using UCommerce.Api;
using UCommerce.EntitiesV2;
using UCommerce.Runtime;
using Uccelerate.Helpers;
using Uccelerate.Services.Model;
using Umbraco.Web.WebApi;
using System.Web.Http;

namespace UccelerateApi
{
    using Basket = Uccelerate.Services.Model.Basket;

    public class BasketRequest
    {
        public string Currency { get; set; }
        public string Culture { get; set; }
        public int SiteId { get; set; }
    }
    public class BasketResponse
    {
        public Basket Basket { get; set; }
    }

    public class BasketController : UmbracoApiController
    {
        [HttpPost]
        public BasketResponse Get(BasketRequest request)
        {
            if (!String.IsNullOrEmpty(request.Culture))
                Thread.CurrentThread.CurrentCulture = new CultureInfo(request.Culture);

            var site = Umbraco.TypedContent(request.SiteId);
            var basketResponse = new BasketResponse();
            var basket = TransactionLibrary.GetBasket(true);
            Currency currency = SiteContext.Current.CatalogContext.CurrentPriceGroup.Currency;

            if (request.Currency != null)
            {
                currency = Currency.FirstOrDefault(x => x.Name == request.Currency);
            }

            PurchaseOrder po = basket.PurchaseOrder;
            var shippingTotal = new Money(po.ShippingTotal.GetValueOrDefault(), currency);
            var paymentTotal = new Money(po.PaymentTotal.GetValueOrDefault(), currency);
            var subTotal = new Money(po.SubTotal.GetValueOrDefault(), currency);
            var taxTotal = new Money(po.TaxTotal.GetValueOrDefault(), currency);
            var discountTotal = new Money(po.DiscountTotal.GetValueOrDefault(), currency);
            var orderTotal = new Money(po.OrderTotal.GetValueOrDefault(), currency);
            var productsTotal = new Money((po.OrderTotal.GetValueOrDefault() - po.ShippingTotal.GetValueOrDefault() - po.PaymentTotal.GetValueOrDefault()), currency);

            if (po.OrderLines.Count == 0)
                orderTotal = new Money(0, currency);

            basketResponse.Basket = new Basket
            {
                FormattedShippingTotal = shippingTotal.ToString(),
                FormattedPaymentTotal = paymentTotal.ToString(),
                SubTotal = po.SubTotal,
                TaxTotal = po.TaxTotal,
                DiscountTotal = po.DiscountTotal,
                OrderTotal = po.OrderTotal,
                TotalItems = po.OrderLines.Sum(l => l.Quantity),
                FormattedSubTotal = subTotal.ToString(),
                FormattedTaxTotal = taxTotal.ToString(),
                FormattedDiscountTotal = discountTotal.ToString(),
                FormattedOrderTotal = orderTotal.ToString(),
                FormattedProductsTotal = productsTotal.ToString(),
                FormattedTotalItems = po.OrderLines.Sum(l => l.Quantity).ToString("#,##"),
                LineItems = new List<LineItem>(),
                Discounts = new List<Uccelerate.Services.Model.Discount>()
            };

            foreach (var line in po.OrderLines)
            {
                var product = CatalogLibrary.GetProduct(line.Sku);

                //lookup to see that product have not been removed
                if (product != null)
                {
                    var variant = product.Variants.FirstOrDefault(x => x.VariantSku == line.VariantSku);
                    var priceWithTax = line.Price + line.VAT;
                    var lineDiscountTotal = line.Discount;
                    var priceExclTax = line.Price - lineDiscountTotal;

                    var url = ProductHelper.GetUrl("", request.Culture, product, site);
                    var imageUrl = ProductHelper.GetPrimaryImage(product);

                    var lineTotal = new Money(line.Total.Value, currency);
                    var priceFormatted = new Money(priceWithTax - lineDiscountTotal, currency);
                    var hasDiscount = (line.UnitDiscount.HasValue && line.UnitDiscount > 0);

                    var variantName = "";

                    if (variant != null)
                    {
                        if (!String.IsNullOrEmpty(variant.GetDescription(request.Culture).DisplayName))
                            variantName = variant.GetDescription(request.Culture).DisplayName;
                        else
                            variantName = variant.Name;
                    }

                    var lineItem = new LineItem
                    {
                        OrderLineId = line.OrderLineId,
                        Quantity = line.Quantity,
                        Sku = line.Sku,
                        VariantSku = line.VariantSku,
                        Url = url,
                        ImageUrl = imageUrl,
                        Price = line.Price,
                        FormattedPrice = priceFormatted.ToString(),
                        ProductName = product.GetDescription(request.Culture).DisplayName,
                        VariantName = variantName,
                        Total = line.Total,
                        FormattedTotal = lineTotal.ToString(),
                        UnitDiscount = line.UnitDiscount,
                        VAT = line.VAT,
                        VATRate = line.VATRate,
                        HasDiscount = hasDiscount
                    };
                    basketResponse.Basket.LineItems.Add(lineItem);

                }
            }

            var discounts = po.Discounts;
            var priceGroup = SiteContext.Current.CatalogContext.CurrentPriceGroup;

            foreach (var discount in discounts)
            {
                var Tax = new Money(discount.AmountOffTotal * priceGroup.VATRate, currency);
                var price = new Money(discount.AmountOffTotal + Tax.Value, currency).ToString();

                basketResponse.Basket.Discounts.Add(new Uccelerate.Services.Model.Discount
                {
                    Name = discount.CampaignItemName,
                    Price = price
                });
            }

            return basketResponse;
        }

        public class AddToBasketRequest
        {
            public int Quantity { get; set; }
            public string Sku { get; set; }
            public string VariantSku { get; set; }
        }

        public class AddToBasketResponse
        {

        }

        [HttpPost]
        public AddToBasketResponse Add(AddToBasketRequest request)
        {
            TransactionLibrary.AddToBasket(request.Quantity, request.Sku, request.VariantSku, addToExistingLine: true, executeBasketPipeline: true);
            return new AddToBasketResponse();
        }

        public class UpdateRequest
        {
            public int NewQuantity { get; set; }
            public int OrderLineId { get; set; }
        }
        public class UpdateResponse
        {
            public LineItem UpdatedLine { get; set; }
        }

        [HttpPost]
        public UpdateResponse Update(UpdateRequest request)
        {
            var updateResponse = new UpdateResponse();

            var line = TransactionLibrary.GetBasket().PurchaseOrder.OrderLines.FirstOrDefault(l => l.OrderLineId == request.OrderLineId);
            TransactionLibrary.UpdateLineItem(request.OrderLineId, request.NewQuantity);
            TransactionLibrary.ExecuteBasketPipeline();

            if (line == null)
            {
                updateResponse.UpdatedLine = new LineItem();
            }

            var currency = SiteContext.Current.CatalogContext.CurrentCatalog.PriceGroup.Currency;
            var lineTotal = new Money(line.Total.Value, currency);

            var product = CatalogLibrary.GetProduct(line.Sku);
            var variant = product.Variants.FirstOrDefault(x => x.VariantSku == line.VariantSku);

            updateResponse.UpdatedLine = new LineItem()
            {
                OrderLineId = line.OrderLineId,
                Quantity = line.Quantity,
                Sku = line.Sku,
                VariantSku = line.VariantSku,
                Price = line.Price,
                ProductName = line.ProductName,
                VariantName = variant != null ? variant.Name : "",
                Total = line.Total,
                FormattedTotal = lineTotal.ToString(),
                UnitDiscount = line.UnitDiscount,
                VAT = line.VAT,
                VATRate = line.VATRate
            };

            return updateResponse;
        }


    }
}
